#This file is part of Tryton.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
import random
from datetime import datetime
from trytond.model import fields
from trytond.pool import PoolMeta

# Size photo default
WIDTH = 200
HEIGHT = 250

__all__ = ['Party']
__metaclass__ = PoolMeta


class Party:
    __name__ = 'party.party'
    birthday = fields.Date('Birthday')
    age = fields.Function(fields.Integer('Age'), 'get_age')
    city_born = fields.Char('City Born')
    photo = fields.Binary('Photo')
    education_level  = fields.Selection([
            ('elementary', 'Elementary'),
            ('secondary', 'Secondary'),
            ('vocational', 'Vocational'),
            ('graduate', 'Graduate'),
            ('postgraduate', 'Postgraduate'),
            ('', ''),
            ], 'Education Level', required=False)
    sex = fields.Selection([
            ('male', 'Male'),
            ('female', 'Female'),
            ('', ''),
            ], 'Sex', required=False)
    marital_status  = fields.Selection([
            ('free_union', 'Free Union'),
            ('married', 'Married'),
            ('single', 'Single'),
            ('divorced', 'Divorced'),
            ('', ''),
            ], 'Marital Status', required=False)
    num_children = fields.Integer('No. Children')
    education = fields.Char('Education')
    responsible_person = fields.Char('Responsible Person')
    phone_responsible_person = fields.Char('Phone Responsible Person')
    nationality = fields.Many2One('party.nationality', 'Nationality')
    health = fields.Text('Health')
    notes = fields.Text('Notes')

    @classmethod
    def __setup__(cls):
        super(Party, cls).__setup__()

    @classmethod
    def create(cls, vals):
        new_vals = []
        for val in vals:
            if val.get('photo'):
                photo =  cls._resize_photo(val.get('photo'))
            new_vals.append(val)
        return super(Party, cls).create(new_vals)

    @classmethod
    def write(cls, employees, vals):
        if 'photo' in vals:
            img_buff = vals.get('photo')
            vals['photo'] = cls._resize_photo(img_buff)
        return super(Party, cls).write(employees, vals)

    def get_age(self, name=None):
        if not self.birthday:
            return
        btd = self.birthday
        res = datetime.now() - datetime(btd.year, btd.month, btd.day)
        res = int(res.days/365.25)
        return res

    @classmethod
    def _resize_photo(cls, img_buff):
        import Image
        from StringIO import StringIO

        io_image = StringIO(img_buff)

        if io_image.len == 0:
            return None

        img = Image.open(io_image)
        width, height = img.size
        
        # To try center original photo for resize it
        factor_resize = float(WIDTH)/width
        height_new = int(height * factor_resize)
        
        # To resize photo to standar size (WIDTH, HEIGHT)
        img_resized = img.resize((WIDTH, height_new),)
        img_resized = img_resized.crop((0, 0, WIDTH, HEIGHT))
        
        io_obj = StringIO()
        img_resized.save(io_obj, 'JPEG')
        img_data_buff = io_obj.getvalue()
        io_obj.close()
        return img_data_buff
